#include once "common.bi"
#include once "srb.bi"
#include once "aspi.bi"

'' --------------------------------------------------------
'' WNASPI32.DLL
'' --------------------------------------------------------

dim shared pfnGetASPI32SupportInfo as function cdecl() as DWORD
dim shared pfnSendASPI32Command as function cdecl( byval as LPSRB ) as DWORD
'' !!! dim shared pfnGetASPI32Buffer as function( byval as PASPI32BUFF ) as DWORD
'' !!! dim shared pfnFreeASPI32Buffer as function( byval as PASPI32BUFF ) as DWORD
'' !!! dim shared pfnTranslateASPI32Address as function( byval as PDWORD, byval as PDWORD ) as DWORD


dim shared hASPIDLL as any ptr
dim shared aspi as ASPITYPE

''
function ASPI_Init() as integer

	if( hASPIDLL ) then
		ASPI_Exit()
	end if

	hASPIDLL = dylibload( "WNASPI32.DLL" )
	if( hASPIDLL <> 0 ) then
		function = TRUE

		pfnGetASPI32SupportInfo = dylibsymbol( hASPIDLL, "GetASPI32SupportInfo" )
		pfnSendASPI32Command = dylibsymbol( hASPIDLL, "SendASPI32Command" )
		'' !!! pfnGetASPI32Buffer = dylibsymbol( hASPIDLL, "GetASPI32Buffer" )
		'' !!! pfnFreeASPI32Buffer = dylibsymbol( hASPIDLL, "FreeASPI32Buffer" )
		'' !!! pfnTranslateASPI32Address = dylibsymbol( hASPIDLL,"TranslateASPI32Address" )

	else
		function = FALSE

		pfnGetASPI32SupportInfo = NULL
		pfnSendASPI32Command = NULL
		'' !!! pfnGetASPI32Buffer = NULL
		'' !!! pfnFreeASPI32Buffer = NULL
		'' !!! pfnTranslateASPI32Address = NULL

	end if

end function

''
function ASPI_Exit() as integer

	pfnGetASPI32SupportInfo = NULL
	pfnSendASPI32Command = NULL
	'' !!! pfnGetASPI32Buffer = NULL
	'' !!! pfnFreeASPI32Buffer = NULL
	'' !!! pfnTranslateASPI32Address = NULL

	if( hASPIDLL ) then
		DylibFree( hASPIDLL )
		hASPIDLL = NULL
	end if

	function = TRUE

end function

''
function GetASPI32SupportInfo() as DWORD
	if( pfnGetASPI32SupportInfo ) then
		function = pfnGetASPI32SupportInfo()
	else
		function = SS_FAILED_INIT
	end if
end function

''
function SendASPI32Command( byval srb as LPSRB ) as DWORD
	if( pfnSendASPI32Command ) then
		function = pfnSendASPI32Command( srb )
	else
		function = SS_FAILED_INIT
	end if
end function

''
function ASPI_GetInfo() as integer	
	
	aspi.supportinfo = GetASPI32SupportInfo()
	aspi.status = HIBYTE(LOWORD(aspi.supportinfo))
	aspi.adaptercount = LOBYTE(LOWORD(aspi.supportinfo))

	if( aspi.status = SS_COMP ) then
		function = TRUE
	else
		function = FALSE
	end if

end function

type ASPIERRMSG
	id as integer
	s as zstring ptr
end type

dim shared tblASPIERRMSG( 0 to ... ) as ASPIERRMSG = _
{ _
	(SS_PENDING                  ,        @"Pending"), _ 
	(SS_COMP                     ,        @"Complete"), _ 
	(SS_ABORTED                  ,        @"Aborted"), _ 
	(SS_ABORT_FAIL               ,        @"Abort failed"), _ 
	(SS_ERR                      ,        @"Error"), _ 
	(SS_ABORTED_QUEUE_FROZEN     ,        @"Aborted queue frozen"), _ 
	(SS_ERR_QUEUE_FROZEN         ,        @"Error queue frozen"), _ 
	(SS_BUFFTOBIG_QUEUE_FROZEN   ,        @"Buffer too big queue frozen"), _ 
	(SS_INVALID_CMD              ,        @"Invalid command"), _ 
	(SS_INVALID_HA               ,        @"Invalid adapter"), _ 
	(SS_NO_DEVICE                ,        @"No device"), _ 
	(SS_ILLEGAL_MODE             ,        @"Illegal mode"), _ 
	(SS_NO_ASPI                  ,        @"No ASPI"), _ 
	(SS_FAILED_INIT              ,        @"Failed Initialization"), _ 
	(SS_ASPI_IS_BUSY             ,        @"ASPI is busy"), _ 
	(SS_BUFFER_TO_BIG            ,        @"Buffer too big"), _ 
	(SS_BUFFER_TOO_BIG           ,        @"Buffer too big"), _ 
	(SS_MISMATCHED_COMPONENTS    ,        @"Mismatched components"), _ 
	(SS_NO_ADAPTERS              ,        @"No apadters"), _ 
	(SS_INSUFFICIENT_RESOURCES   ,        @"Insufficient resources") _ 
}

''
function ASPI_GetErrorMsg( byval statuscode as byte ) as string

	dim msg as string

	for i as integer = lbound(tblASPIERRMSG) to ubound(tblASPIERRMSG)
		if( cbyte(tblASPIERRMSG(i).id) = statuscode ) then
			msg = *tblASPIERRMSG(i).s + " (0x" + hex( statuscode ) + ")"
			exit for
		end if
	next

	if( msg = "" ) then
		msg = "Unrecognized Error (0x" + hex( statuscode ) + ")"
	end if

	function = msg

end function

''
function ASPI_ScanBus _
	( _
		byval dtype as integer, _
		byval dinfo as ASPIDEVICEINFO ptr, _
		byval cb as ASPISCANBUSCB _
	) as integer

	dim ret as integer

	function = FALSE

	if( dinfo = NULL ) then
		exit function
	end if

	if( cb = NULL ) then
		exit function
	end if

	if( hASPIDLL = NULL ) then
		exit function
	end if

	if( aspi.adaptercount <= 0 ) then
		exit function
	end if

	for iAdapter as integer = 0 to aspi.adaptercount - 1

		dim maxdevice as integer = 0

		scope
			dim srb as SRB_HAInquiry

			srb.SRB_Cmd = SC_HA_INQUIRY
			srb.SRB_HaID = iAdapter
			srb.SRB_Flags = 0
			srb.SRB_Hdr_Rsvd = 0

			ret = SendASPI32Command( @srb )
			if( srb.SRB_status <> SS_COMP ) then
				continue for
			end if

			maxdevice = srb.HA_MaxTargets
			if(( maxdevice <=0 ) or (maxdevice > 8 )) then
				maxdevice = 8
			end if
		
		end scope

		for idevice as integer = 0 to maxdevice - 1
			for iLun as integer = 0 to 7

				dim srb as SRB_GetDevType

				clear srb,,sizeof(srb)
				srb.SRB_Cmd = SC_GET_DEV_TYPE
				srb.SRB_HaID = iAdapter
				srb.SRB_Target = iDevice
				srb.SRB_Lun = iLun
    
				ret = SendASPI32Command( @srb )
				if( srb.SRB_Status <> SS_COMP ) Then
					continue for
				end if

				if( srb.SRB_DeviceType = dtype ) then
					dinfo->adapter = iAdapter
					dinfo->device = iDevice
					dinfo->lun = iLun
					if( cb(dinfo) <> 0 ) then
						exit for, for, for
					end if
				end if

			next
		next
	next

	function = TRUE

end function

