#include once "windows.bi"
#include once "win/winsock2.bi"
#include once "logtool.bi"
#include once "sockerr.bi"
#include once "socktool.bi"
#include once "cddb.bi"
#include once "telnet.bi"

#define length(_x) LEN((_x))

const CRLF = chr(13,10)

''
function CDDB_send _
	( _
		byval s as SOCKET, _
		byref cmd as string _
	) as integer

	logmsg( "SEND: " + cmd  )

	function = TELNET_send( s, cmd )

end function

''
function CDDB_readline _
	( _
		byval s as SOCKET, _
		byref text as string _
	) as integer

	function = TELNET_readline( s, text )

end function

'' 
function CDDB_getresponse _
	( _
		byref response as string _
	) as integer

	'' Get Reponse code
	dim i as integer = instr( response, " " )
	if( i > 0 ) then
		function = val( left(response, i) )
	else
		function = val( response )
	end if

end function

''
function CDDB_parsediscinfo _
	( _
		byref text as string, _
		byref disc as CDDB_Disc_Info _
	) as integer

	dim dcateg as string
	dim discid as string
	dim dtitle as string
	dim t as string
	dim i as integer

	t = trim(text)

	i = instr( t, " " )
	if( i > 0 ) then
		dcateg = left( t, i-1 )
		t = ltrim( mid( t, i+1 ))
	else
		dcateg = t
		t = ""
	end if

	i = instr( t, " " )
	if( i > 0 ) then
		discid = left( t, i-1 )
		t = ltrim( mid( t, i+1 ))
	else
		discid = t
		t = ""
	end if

	i = instr( t, " " )
	if( i > 0 ) then
		dtitle = left( t, i-1 )
		t = ltrim( mid( t, i+1 ))
	else
		dtitle = t
		t = ""
	end if

	disc.dcateg = dcateg
	disc.discid = discid
	disc.dtitle = dtitle

	function = TRUE

end function

''
function CDDB_readresponse _
	( _
		byval s as SOCKET _
	) as integer

	dim x as string
	
	if( CDDB_readline( s, x ) >= 0 ) then
		function = CDDB_getresponse( x )
	else
		function = -1
	end if

end function

''
function CDDB_recvlist _
	( _
		byval s as SOCKET, _
		x() as string _
	) as integer

	dim ret as integer
	dim n as integer
	dim text as string
	dim i as integer

	n = 0

	do
		ret = CDDB_readline( s, text )
		if( ret >= 0 ) then
			if( text = "." ) then
				exit do
			else
				do
					i = instr( text, any chr(10) + chr(13) )
					if( i > 0 ) then
						x(n) = left( text, i-1 )
						text = mid( text, i+1 )
						n += 1
					else
						x(n) = text
						n += 1
						exit do
					end if
				loop
			end if
		else
			exit do
		end if
	loop

	function = n

end function

''
function CDDB_connect _
	( _
		byval s as SOCKET, _
		byref cddb as CDDB_Connect_Info _
	) as integer

	dim cmd as string
	dim ret as integer

	function = FALSE

	'' Initial response on connection

	ret = CDDB_readresponse( s )
	select case ret
	case 200
		'' OK, read/write
	case 201
		'' OK, read only
	case else
		exit function
	end select

	'' cddb hello username hostname clientname version

	cmd = "cddb hello" 
	cmd &= " " & cddb.username 
	cmd &= " " & cddb.hostname 
	cmd &= " " & cddb.clientname
	cmd &= " " & cddb.version
	cmd &= CRLF

	ret = CDDB_send( s, cmd )
	ret = CDDB_readresponse( s )
	select case ret
	case 200
	case else
		exit function
	end select

	''  proto 5

	cmd = "proto 5" + CRLF

	ret = CDDB_send( s, cmd )
	ret = CDDB_readresponse( s )

	select case ret
	case 200, 201, 502
		print "Response: "; ret
	case else
		exit function
	end select

	function = TRUE

end function

''
function CDDB_query _
	( _
		byval s as SOCKET, _
		byref query as CDDB_Query_Request, _
		byref result as CDDB_Query_Result _
	) as integer

	dim cmd as string
	dim ret as integer
	dim text as string
	dim disc as CDDB_Disc_Info

	function = FALSE

	'' cddb query discid ntracks tack_offsets... total_seconds

	cmd = query.querystring & CRLF

	ret = CDDB_send( s, cmd )
	if( CDDB_readline( s, text ) > 0 ) then

		ret = CDDB_getresponse( text )
		select case ret
		case 200
			'' exact match
			dim i as integer 
			
			i = instr( text, " " )
			text = ltrim( mid( text, i+1 ))

			function = CDDB_parsediscinfo( text, disc )

			result.ndiscs = 1
			result.discs(0) = disc

		case 210, 211
			'' multiple matches
			dim x(0 to 999) as string
			dim n as integer

			n = CDDB_recvlist( s, x() )
			if( n > 0 ) then
				result.ndiscs = 0
				for i as integer = 0 to n - 1
					if( CDDB_parsediscinfo( x(i), disc ) ) then
						result.discs(result.ndiscs) = disc
						result.ndiscs += 1
					end if
				next
				if( result.ndiscs > 0 ) then
					function = TRUE
				end if
			end if

		case else
			function = FALSE

		end select

	else
		function = FALSE

	end if

end function

''
function CDDB_quit _
	( _
		byval s as SOCKET _
	) as integer

	dim cmd as string
	dim ret as integer

	function = FALSE

	'' QUIT

	cmd = "quit"
	cmd &= CRLF

	ret = CDDB_send( s, cmd )
	ret = CDDB_readresponse( s )

	select case ret
	case 230
		'' Closing connection.  Goodbye.
	case else
		exit function
	end select

	function = TRUE

end function

''
function CDDB_read _
	( _
		byval s as SOCKET, _
		byref request as CDDB_Read_Request, _
		byref result as CDDB_Read_Result _
	) as integer

	dim cmd as string
	dim ret as integer
	dim n as integer
	dim x(0 to 999) as string

	function = FALSE

	'' cddb read categ discid

	cmd = "cddb read"
	cmd &= " " & request.dcateg
	cmd &= " " & request.discid	
	cmd &= CRLF

	ret = CDDB_send( s, cmd )
	ret = CDDB_readresponse( s )

	select case ret
	case 210
		dim ntracks as integer
		dim i as integer
		dim itrack as integer
		dim w as string, v as string

		n = CDDB_recvlist( s, x() )

		result.record = ""
		for iline as integer = 0 to n - 1
			result.record += x(iline) + chr(13) + chr(10)
		next
		
		result.ntracks = 0

		for iline as integer = 0 to n - 1
			x(iline) = trim(x(iline))
			i = instr( x(iline), "=" )
			if( i > 0 ) then
				w = rtrim(left(x(iline),i-1))
				v = ltrim(mid(x(iline),i+1))
				if( ucase(left(w,6)) = "TTITLE" ) then
					itrack = val(mid(w,7))
					if( (itrack >= 0) and (itrack < 99) ) then
						if( itrack + 1 > result.ntracks ) then
							result.ntracks = itrack + 1
						end if
						result.tracks(itrack).ttitle = v
					end if
				end if
			end if
		next


	case else
		exit function
	end select

	function = TRUE

end function
