#include once "cdrom.bi"
#include once "windows.bi"
#include once "cdrom_ioctl.bi"

type CDROMDEVICE
	drive_num as integer
	rootpath as string
	path as string
end type

type TOC_TRACK field = 1
    reserved1 as ubyte
    adr as ubyte
    track as ubyte
    reserved2 as ubyte
    addr(0 to 3) as ubyte
end type

type TOC_CDROM field = 1
    toclen(0 to 1) as ubyte
    firsttrack as ubyte
    lasttrack as ubyte
    tracks(0 to 99) as toc_track
end type

/'
#define MAXIMUM_NUMBER_TRACKS 100

typedef struct _TRACK_DATA
{
  UCHAR Reserved;
  UCHAR Control : 4;
  UCHAR Adr : 4;
  UCHAR TrackNumber;
  UCHAR Reserved1;
  UCHAR Address[4];
} TRACK_DATA;


typedef struct _CDROM_TOC
{
  UCHAR Length[2];
  UCHAR FirstTrack;
  UCHAR LastTrack;
  TRACK_DATA TrackData[MAXIMUM_NUMBER_TRACKS];
} CDROM_TOC;
'/

dim shared ncdromdevices as integer = 0
dim shared cdromdevices(0 to 15) as CDROMDEVICE
dim shared cdtoc as TOC_CDROM

'' --------------------------------------------------------

function GetDriveTypeString _
	( _
		byval drive_type as UINT _
	) as string

	dim x as string

	select case drive_type
	case DRIVE_UNKNOWN 
		x = "Unknown"
 
	case DRIVE_NO_ROOT_DIR 
		x = "No Root Directory"
 
	case DRIVE_REMOVABLE 
		x = "Removeable"
 
	case DRIVE_FIXED
		x = "Fixed" 
 
	case DRIVE_REMOTE 
		x = "Remote"
 
	case DRIVE_CDROM
		x = "CD-ROM drive"
 
	case DRIVE_RAMDISK 
		x = "RAM disk"
 
	case else
		x = "Unrecognized drive type"

	end select

	function = x & " (0x" & hex(drive_type) & ")"

end function

''
function CDROM_ScanDevices() as integer

''	print "Scanning for devices ..."

	dim drives as DWORD = GetLogicalDrives()
	for i as integer = 0 to 31
		if( ((2 ^ i) and drives ) <> 0 ) then
			dim path as zstring * 4 = chr( i + 65 ) & ":\"
			dim rootpath as zstring * 8 = $"\\.\" & chr( i + 65 ) & ":"
			dim as UINT drive_type = GetDriveType( path )
			dim dt as string = GetDriveTypeString( drive_type )

			if( drive_type = DRIVE_CDROM ) then
				print rootpath & " " & path & " " & dt

				cdromdevices( ncdromdevices ).drive_num = i
				cdromdevices( ncdromdevices ).rootpath = rootpath
				cdromdevices( ncdromdevices ).path = path
				ncdromdevices += 1

			end if
			
		end if
	next
	print "Found " & ncdromdevices & " device(s)"

	function = ncdromdevices

end function

function CDROM_ReadTOC _
	( _
		byref rootpath as const string _
	) as integer

	function = 0

	dim as HANDLE hCD = NULL
	dim t as TOC_CDROM
	dim bret as BOOL
	dim bytes as DWORD = 0
	cdtoc = t

	hCD = CreateFile( rootpath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL )

	if( hCD =  INVALID_HANDLE_VALUE ) then
		exit function
	end if

	bret = DeviceIoControl( hCD, IOCTL_CDROM_READ_TOC, NULL, 0, @t, sizeof(TOC_CDROM), @bytes, 0 )

	CloseHandle( hCD )

	cdtoc = t

	if( bret <> 0 ) then
		function = cdtoc.lasttrack - cdtoc.firsttrack
	end if

end function

''
function CDROM_ReadTOCByIndex _
	( _
		byval index as integer _
	) as integer

	if( (index >= 0) and (index < ncdromdevices)) then
		function = CDROM_ReadTOC _
			( _
				cdromdevices(index).rootpath _
			)
	else
		function = FALSE
	end if

end function

''
function CDROM_GetNumTracks() as integer
	if( cdtoc.firsttrack <> 0 and cdtoc.lasttrack <> 0 ) then
		function = cdtoc.lasttrack - cdtoc.firsttrack + 1
	else
		function = 0
	end if
end function

''
function CDROM_ListTOC() as integer

	if( cdtoc.firsttrack <> 0 and cdtoc.lasttrack <> 0 ) then
		dim n as integer
		n = cdtoc.lasttrack - cdtoc.firsttrack + 1
		function = n
		for i as integer = 1 to n + 1
			with cdtoc.tracks(i - cdtoc.firsttrack)
				print .adr, hex(.track), 
				print right( "00" & .addr(1), 2);
				print ":";
				print right( "00" & .addr(2), 2);
				print ":";
				print right( "00" & .addr(3), 2)
			end with
		next
	else
		function = 0
	end if

end function

''
function CDROM_GetTrackOffset( byval i as integer ) as integer

	function = ( cint(cdtoc.tracks(i).addr(1)) * 60 _
				+ cint(cdtoc.tracks(i).addr(2))) * 75 _
				+ cint(cdtoc.tracks(i).addr(3))

end function

''
function CDROM_GetTotalLengthSeconds() as integer

   function = CDROM_GetTrackOffset( CDROM_GetNumTracks() ) \ 75
   
end function



'' --------------------------------------------------------

type TOCOFFSET
	min as integer
	sec as integer
	fra as integer
end type

dim shared tracks(0 to 99) as TOCOFFSET

''
function cddb_sum( byval n as integer ) as integer

	dim ret as integer = 0

	while (n > 0)
		ret = ret + (n mod 10)
		n = n \ 10
	wend

	function = ret

end function

''
function CDROM_get_cddb_discid() as ulongint

	dim as integer i = 0, t = 0, n = 0, tot_trks = 0
	
	tot_trks = CDROM_GetNumTracks()

	for i as integer = 0 to tot_trks
		tracks(i).min = cdtoc.tracks(i).addr(1)
		tracks(i).sec = cdtoc.tracks(i).addr(2)
		tracks(i).fra = cdtoc.tracks(i).addr(3)
	next

	while (i < tot_trks)
		n = n + cddb_sum((tracks(i).min * 60) + tracks(i).sec)
		i += 1
	wend

	t = ((tracks(tot_trks).min * 60) + tracks(tot_trks).sec) _
	    - ((tracks(0).min * 60) + tracks(0).sec)

	function = ((n mod &hff) shl 24) or (t shl 8) or (tot_trks)

end function

''
function CDROM_get_cddb_query_string() as string

	dim x as string
	dim n as integer

	n = CDROM_GetNumTracks()

	x = "cddb query"
	x &= " " & lcase(hex(CDROM_get_cddb_discid(),8))
	x &= " " & n
	for i as integer = 0 to n-1
		x &= " " & CDROM_GetTrackOffset(i)
	next
	x &= " " & CDROM_GetTotalLengthSeconds()
	
	function = x

end function
