#include once "windows.bi"
#include once "win/winsock2.bi"
#include once "logtool.bi"
#include once "sockerr.bi"
#include once "socktool.bi"
#include once "cdrom.bi"
#include once "cddb.bi"
#include once "freedb.bi"

dim shared as SOCKET s
dim shared as integer connected
dim shared as integer validated

''
function FREEDB_Connect _
	( _
		byref conn as FREEDB_CONNECT_INFO _
	) as integer

	dim as SOCKADDR_IN remoteip
	dim as SOCKADDR_IN localip
	dim as WSADATA wsaInformation
	dim as HOSTENT ptr hp

	if( WSAStartup( &h202, @wsaInformation) = SOCKET_ERROR ) then
		logmsg( "Error: " & WSAGetLastError() )
		exit function
	end if

	''
	if( MakeInAddr( GetLocalHostName(), 0, @localip ) = FALSE ) then
		logmsg( "Unable to resolve local host name" )
		WSACleanup()
		exit function
	end if

	logmsg( "Local host is " & GetLocalHostName() & " (" & GetAddrInText(@localip) & ")" )

	''
	if( MakeInAddr( conn.remote_hostname, conn.remote_port, @remoteip ) = FALSE ) then
		logmsg( "Unable to resolve '" & conn.remote_hostname & "'" )
		WSACleanup()
		exit function
	end if

	logmsg( "Connecting to " & conn.remote_hostname & " (" & GetAddrInText(@remoteip) & ")" )

	if( ConnectToHostTCP( @localip, @remoteip, @s ) = FALSE ) then 
		logmsg( "Unable to connect to " & conn.remote_hostname )
		WSACleanup()
		exit function
	end if

	connected = TRUE
	logmsg( "Connected" )

	dim cddb as CDDB_Connect_info
	cddb.username = conn.username
	cddb.hostname = conn.hostname
	cddb.clientname = "FreeBASIC-cddir"
	cddb.version = "1.0"

	if( CDDB_connect( s, cddb ) ) then
		validated = TRUE
		function = TRUE
	else
		FREEDB_Disconnect()
	end if

end function

''
sub FREEDB_Disconnect()
	
	logmsg( "Disconnecting" )
	
	if( validated ) then
		CDDB_quit( s )
	end if
	
	if( connected ) then
		closesocket( s )  
		WSACleanup()
	end if

	validated = FALSE
	connected = FALSE
	s = cast( SOCKET, NULL )

end sub

''
function FREEDB_Query _
	( _
		byref query as FREEDB_QUERY_REQUEST, _
		byref result as FREEDB_QUERY_RESULT _
	) as integer

	function = FALSE

	dim q as CDDB_Query_Request
	dim r as CDDB_Query_Result

	q = query

	if( CDDB_query( s, q, r ) ) then
		result = r
		function = TRUE
	end if

end function

''
function FREEDB_Read _
	( _
		byref request as FREEDB_READ_REQUEST, _
		byref result as FREEDB_READ_RESULT _
	) as integer

	dim q as CDDB_Read_Request
	dim r as CDDB_Read_Result

	q = request

	function = FALSE

	if( CDDB_read( s, q, r ) ) then
		result = r
		function = TRUE
	end if

end function

''
function FREEDB_GetDiscography( byref file as string ) as integer

	print
	print "Connecting to server ... "

	dim conn as FREEDB_CONNECT_INFO
	conn.remote_hostname = "freedb.freedb.org"
	conn.remote_port = 8880
	conn.username = "fbuser"
	conn.hostname = "localhost"

	function = FALSE

	if( FREEDB_Connect( conn ) ) then

		print "Connected. OK."

		dim qreq as FREEDB_QUERY_REQUEST
		dim discset as FREEDB_QUERY_RESULT
		
		qreq.discid = lcase(hex(CDROM_get_cddb_discid(),8))
		qreq.querystring = CDROM_get_cddb_query_string()

		dim result as integer

		result = FREEDB_Query( qreq, discset )

		print "Result = " & result

		if( result <> 0 ) then
			dim ndiscs as integer
	
			ndiscs = discset.ndiscs

			print "Found " & ndiscs

			for idisc as integer = 0 to ndiscs - 1
			
				dim rreq as FREEDB_READ_REQUEST
				dim disc as FREEDB_READ_RESULT
				
				rreq.dcateg = discset.discs(idisc).dcateg
				rreq.discid = discset.discs(idisc).discid

				print rreq.dcateg, rreq.discid

				if( FREEDB_Read( rreq, disc ) ) then

					if( file > "" ) then

						dim h as integer = freefile
						dim file2 as string
						file2 = file & idisc & ".discid"
						open file2 for output as #h
						close #h
						open file2 for binary as #h
						put #h,,disc.record
						close #h

					end if

					if( disc.ntracks > 0 ) then
						print "Tracks: "; disc.ntracks
						for itrack as integer = 0 to disc.ntracks - 1
							print str(itrack+1) & " " & disc.tracks(itrack).ttitle
						next

					else
						print "No tracks"

					end if

				end if
			next

			function = TRUE

		end if

		FREEDB_Disconnect()

	else

		print "Connection Failed!"

	end if

end function
